/*
 * Copyright (c) 2016, 2017 Jonas 'Sortie' Termansen.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * net/ip.h
 * Internet Protocol Version 4.
 */

#ifndef SORTIX_NET_IP_H
#define SORTIX_NET_IP_H

#include <netinet/if_ether.h>
#include <netinet/in.h>

#include <sortix/kernel/packet.h>

namespace Sortix {
namespace IP {

uint16_t ipsum(const void* bufptr, size_t size);
uint16_t ipsum_word(uint16_t sum, uint16_t word);
uint16_t ipsum_buf(uint16_t sum, const void* bufptr, size_t size);
uint16_t ipsum_finish(uint16_t sum);
void Handle(Ref<Packet> pkt,
            const struct ether_addr* src,
            const struct ether_addr* dst,
            bool dst_broadcast);
bool Send(Ref<Packet> pkt,
          const struct in_addr* src,
          const struct in_addr* dst,
          uint8_t protocol,
          unsigned int ifindex,
          bool broadcast);
bool GetSourceIP(const struct in_addr* src,
                 const struct in_addr* dst,
                 struct in_addr* sendfrom,
                 unsigned int ifindex,
                 size_t* mtu = NULL);
Ref<Inode> Socket(int type, int protocol);

} // namespace IP
} // namespace Sortix

#endif
