/*
 * Copyright (c) 2014, 2022 Jonas 'Sortie' Termansen.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * stdio/asprintf.c
 * Prints a string to a newly allocated buffer.
 */

#include <stdarg.h>
#include <stdio.h>

#ifdef __TRACE_ALLOCATION_SITES
int asprintf_trace(struct __allocation_site* allocation_site,
                   char** restrict result_ptr,
                   const char* restrict format,
                   ...)
#else
int asprintf(char** restrict result_ptr,
             const char* restrict format,
             ...)
#endif
{
	va_list list;
	va_start(list, format);
#ifdef __TRACE_ALLOCATION_SITES
	int result = vasprintf_trace(allocation_site, result_ptr, format, list);
#else
	int result = vasprintf(result_ptr, format, list);
#endif
	va_end(list);
	return result;
}
